#!/usr/bin/env python

import argparse
import config
import os
import rpclib
import sys


def main():
  parser = argparse.ArgumentParser(description='pspy rpc server')
  parser.add_argument(
      '--socket', help=(
          'the socket where to read requests from (absolute path from within '
          'the jail)'), required=True)
  parser.add_argument(
      '--service-provider', help='the service provider name',
      required=True)
  parser.add_argument(
      '--services', nargs='+', help=(
          'the services to provide (in the form module.class)'),
      required=True)
  parser.add_argument(
      '--uid', help='the uid under which to run this service',
      required=True)
  parser.add_argument(
      '--gid', help='the gid under which to run this service',
      required=True)
  parser.add_argument(
      '--jail-dir', help='the jail-dir where to jail this service',
      required=True)
  parser.add_argument(
      '--additional-groups', nargs='*', help='the list of additional group ids',
      required=True)

  dw = lambda status: lambda msg: sys.stdout.write('[%s] %s\n' % (status, msg))
  dw_fail = dw('FAILURE')
  dw_success = dw('SUCCESS')
  dw_info = dw('INFO')

  args = parser.parse_args()
  dw_info('Jailing myself to %s' % args.jail_dir)
  os.chdir(args.jail_dir)
  os.chroot(args.jail_dir)
  dw_success('Done, jailed myself to %s' % args.jail_dir)

  gid = int(args.gid)
  dw_info('Setting gid to %s' % gid)
  os.setgid(gid)
  dw_success('Set gid to %s' % gid)

  add_group_ids = [int(x) for x in args.additional_groups]
  dw_info('Setting additional groups to %s' % add_group_ids)
  os.setgroups(add_group_ids)
  dw_success('Set additional groups to %s' % add_group_ids)

  uid = int(args.uid)
  dw_info('Setting uid to %s' % uid)
  os.setuid(uid)
  dw_success('Set uid to %s' % uid)

  sys.path.append(os.getcwd())
  config.server = args.service_provider

  # Start the server.
  sock = os.path.join(args.socket, 'sock')
  dw_info('Starting the RpcServer on socket %s' % sock)
  rpc_server = rpclib.RpcServer(args.services)
  rpc_server.run_sockpath_fork(sock)


if __name__ == '__main__':
  main()
